==> coventry-0.8.1.tar.gz: coventry-0.8.1/src/nonce.hpp <==
/*
 * Copyright (C) 2020 Tycho Softworks.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef NONCE_HPP_
#define NONCE_HPP_

#include <moderncli/sync.hpp>

#include <atomic>
#include <string>
#include <cstring>

#include <arpa/inet.h>

#include "server.hpp"

namespace server {
class nonce_t final {
public:
    explicit nonce_t(const std::string_view& from);  // from string...
    nonce_t();                        // generate new nonce

    auto operator=(const std::string_view& from) -> nonce_t&;

    inline auto operator==(const nonce_t& other) const {
        return memcmp(data(), other.data(), 16) == 0;
    }

    inline auto operator!=(const nonce_t& other) const {
        return memcmp(data(), other.data(), 16) != 0;
    }

    inline auto data() const noexcept -> const uint8_t *{
        return &data_.buffer[0];
    }

    inline auto view() const {
        return std::string_view(key_);
    }

    inline auto hex() const {
        return key_.c_str();
    }

    inline auto timestamp() const -> uint32_t {
        return htonl(data_.timestamp);
    }

    inline auto created() const {
        return created_;
    }

    inline auto size() const {
        return sizeof(data_.buffer);
    }

    auto hash() const -> size_t;

    static auto current_timestamp() -> uint32_t;

private:
    union {
        uint8_t buffer[16];
#pragma pack(push, 1)           // possibly unneeded...
        struct alignas (1) {
            uint8_t random_iv[8];       // from truly random...
            uint32_t timestamp;        // local time clock, low 32 bits...
            uint32_t sequence;         // from atomic sequence generator
        };
#pragma pack(pop)
    } data_;
    std::string key_;
    sync_timepoint created_;
    static_assert (sizeof(data_) == 16U, "Nonce must be 16 bytes");
};
} // end namespace

namespace std {
template<>
struct hash<server::nonce_t> {
    auto operator()(const server::nonce_t& key) const {
        return key.hash();
    }
};
}
#endif

==> coventry-0.8.6.tar.gz: coventry-0.8.6/src/nonce.hpp <==
/*
 * Copyright (C) 2020 Tycho Softworks.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef NONCE_HPP_
#define NONCE_HPP_

#include "server.hpp"       // IWYU pragma: keep

#include <moderncli/sync.hpp>

#include <arpa/inet.h>

namespace server {
class nonce_t final {
public:
    explicit nonce_t(const std::string_view& from);  // from string...
    nonce_t();                        // generate new nonce

    auto operator=(const std::string_view& from) -> nonce_t&;

    auto operator==(const nonce_t& other) const {
        return memcmp(data(), other.data(), 16) == 0;
    }

    auto operator!=(const nonce_t& other) const {
        return memcmp(data(), other.data(), 16) != 0;
    }

    auto data() const noexcept -> const uint8_t *{
        return &data_.buffer[0];
    }

    auto view() const {
        return std::string_view(key_);
    }

    auto hex() const {
        return key_.c_str();
    }

    auto timestamp() const -> uint32_t {
        return htonl(data_.timestamp);
    }

    auto created() const {
        return created_;
    }

    auto size() const {
        return sizeof(data_.buffer);
    }

    auto hash() const -> size_t;

    static auto current_timestamp() -> uint32_t;

private:
    union {
        uint8_t buffer[16];
#pragma pack(push, 1)           // possibly unneeded...
        struct alignas (1) {
            uint8_t random_iv[8];       // from truly random...
            uint32_t timestamp;        // local time clock, low 32 bits...
            uint32_t sequence;         // from atomic sequence generator
        };
#pragma pack(pop)
    } data_;
    std::string key_;
    sync_timepoint created_;
    static_assert (sizeof(data_) == 16U, "Nonce must be 16 bytes");
};
} // end namespace

namespace std {
template<>
struct hash<server::nonce_t> {
    auto operator()(const server::nonce_t& key) const {
        return key.hash();
    }
};
}
#endif
